"""
Acoustics processing (:mod:`acous.acoustics`)
=======================================

Wavelets
--------


Sound Pressure Level (SPL)
--------------------------
   from_pressure_to_spl - Calculate sound pressure level from sound pressure.
   from_spl_to_pressure - Calculate sound pressure from sound pressure level.
   from_spl_to_oaspl - Calculate overall sound pressure level from sound pressure level.

"""
from .spl import *


import numpy as np

def hanning(M):
    """
    Return the Hanning window.

    Parameters
    ----------
        M : int
            Number of points in the output window.
            If zero or less, an empty array is returned.

    Returns
    -------
        out : ndarray, shape(M,)
            The window, with the maximum value normalized to one 
            (this maximum value `one` appears only if `M` is odd).

    References
    ----------
        [1] https://numpy.org/doc/stable/reference/generated/numpy.hanning.html
        [2] https://en.wikipedia.org/wiki/Window_function

    """
    if M < 1:
        return np.array([])
    if M == 1:
        return np.ones(1, dtype=float)
    n = np.arange(0,M,1)

    return 0.5-0.5*np.cos(2*np.pi*n/(M-1)) #return np.square(np.sin(np.pi*n/(M-1)))


import numpy as np
from acous import fft

def periodogram(a, fs=1.0, scaling='density'):
    """
    Estimate power spectral density using a periodogram of one-dimensional real value series.

    Parameters
    ----------
        a : array_like
            Input array, should be real input.
        fs : float, optional
            Sampling frequency of the time series. Defaults to 1.0.
        scaling : { 'density', 'spectrum' }, optional
            Selects between computing the power spectral density ('density')
            where `Pxx` has units of V**2/Hz (electric signal) or P**2/Hz (acoustic signal)
            and computing the power spectrum ('spectrum') where `Pxx` has units of V**2,
            if `x` is measured in V and `fs` is measured in Hz. Defaults to 'density'.

    Returns
    -------
        f : ndarray
            Array of length [0, Fs/2] = [0, 1/(2*d)] containing the postive frequencies.
        |Pxx| : ndarray
            Single-sided power spectral density (absolute value)

    Notes
    -----
        Sxx : ndarray
            Double-sided power spectral density

            if scaling == 'density' # ref [1]
                Sxx = (1/(fs*N))*(|A|^2)
            if scaling == 'spectrum' # ref [2] (1/N)*(|A|^2) but here we refer scipy and powerflow
                Sxx = (1/(N*N))*(|A|^2)
            in which fs is sampling frequency, dt is sampling interval, and fs = 1/dt with dt = d is the samping interval
            N is the number of total sampling points, and |A| = abs(fft(a)) = A*conjugate(A)

        We need to redefine the frequency of the fft() according to the following reason:
        The values in the result follow so-called “standard” order: If A = fft(a, n),
        then A[0] contains the zero-frequency term (the sum of the signal), which is
        always purely real for real inputs. Then A[1:n/2] contains the positive-frequency
        terms, and A[n/2+1:] contains the negative-frequency terms, in order of
        decreasingly negative frequency. For an even number of input points, A[n/2]
        represents both positive and negative Nyquist frequency, and is also purely
        real for real input. For an odd number of input points, A[(n-1)/2] contains
        the largest positive frequency, while A[(n+1)/2] contains the largest negative
        frequency. # ref [3]

        So here, for an even number of input points, we add A[n/2] and the Nyquist frequency.

    References
    ----------
        [1] The class of EAE298 by Prof. Seongkyu Lee at University of California, Davis
        [2] Michael Cerna, Audrey Harvey. The fundamentals of FFT-based signal analysis and measurement.
            Application Note 041, National Instruments, 2000.
        [3] https://numpy.org/doc/stable/reference/routines.fft.html

    """

    a = np.asarray(a, dtype='float') # make sure input array contains no complex numbers
    N = len(a)

    A = fft.fft(a, 'scipy')

    if scaling == 'density':
        scale = 1.0/(fs*N)
    elif scaling == 'spectrum':
        scale = 1.0/(N*N)
    else:
        raise ValueError(f"Unknown scaling value '{scaling}'; should be 'density' or 'spectrum'.")

    Sxx = scale*(A*np.conjugate(A))

    if N%2 == 0:
        N_pos = N//2
        f = (fs/N)*np.arange(0, N_pos+1, dtype=int)
        Pxx = np.empty(N_pos+1, complex)

        Pxx[0] = Sxx[0] # the DC value at 0 [Hz] is usually discarded in scipy and PowerFLOW
        Pxx[1:N_pos] = 2.0*Sxx[1:N_pos]
        Pxx[N_pos] = Sxx[N_pos]
    
    else:
        N_pos = (N+1)//2
        f = (fs/N)*np.arange(0, N_pos, dtype=int)
        Pxx = np.empty(N_pos, complex)

        Pxx[0] = Sxx[0]
        Pxx[1:N_pos] = 2.0*Sxx[1:N_pos]

    return f, np.abs(Pxx)


##==========~~~~~~~~~~==========~~REMARK~~==========~~~~~~~~~~==========##
## Code:    Python Plot Style
## Author:  Wende Li
## Email:   wende.li@outlook.com
## Version:
## >>       1.0: plot setup completed based on python matplotlib
## General:
## >>       This plot style is suitable for pictures in paper publication
## >>       Install matplotlib: python -m pip install -U matplotlib
## >>       Reference: matplotlib.org
## >>       Reference: https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_csv.html
##==========~~~~~~~~~~==========~~~~~~~~~~==========~~~~~~~~~~==========##


##==========~~~~~~~~~~==========~~IMPORT~~==========~~~~~~~~~~==========##
## DEPENDENT MODULES
import os
import sys
import glob
import numpy as np
import pandas as pd
import matplotlib as mpl
##==========~~~~~~~~~~==========~~~~~~~~~~==========~~~~~~~~~~==========##


##==========~~~~~~~~~~==========~FUNCTION~==========~~~~~~~~~~==========##
def plot_setup(font_size='8'):
    ''' define font size of generated pictures
    '''
    if not isinstance(font_size,str):
        print(_ERROR_+' font_size '+f'{font_size}')
        sys.exit(_STR_ERROR_)

    font_name = 'sans-serif'

    mpl.rcParams['font.family']         =   font_name   # default text font name
    mpl.rcParams['font.size']           =   font_size   # default text font size
    mpl.rcParams['axes.titlesize']      =   font_size   # fontsize of the title
    mpl.rcParams['axes.labelsize']      =   font_size   # fontsize of the labels
    mpl.rcParams['xtick.labelsize']     =   font_size   # fontsize of the x ticks
    mpl.rcParams['ytick.labelsize']     =   font_size   # fontsize of the y ticks

    mpl.rcParams['legend.fontsize']     =   font_size   # fontsize of the legend
    mpl.rcParams['legend.frameon']      =   False       # frame of the legend
    mpl.rcParams["legend.labelspacing"] =   0.2         # space of legend entries, font-size units

    mpl.rcParams['lines.linewidth']     =   1.2
    mpl.rcParams['lines.markersize']    =   4
    mpl.rcParams['lines.linestyle']     =   '-'

    mpl.rcParams['figure.figsize']      =   (4, 3)      # figure size (width, height) in inches
    mpl.rcParams['figure.dpi']          =   100         # dots per inch

    mpl.rcParams['axes.autolimit_mode'] = 'data'        # offset of axes
    mpl.rcParams['axes.xmargin']        = 0.
    mpl.rcParams['axes.ymargin']        = 0.

def plot_reset():
    ''' reset the plot configuration
    '''
    plot_setup(font_size='8')

def read_data(file_name,use_cols=None,d_type='a',skip_rows=0,skip_blank=True,clean_data=False):
    ''' read data from a file
        file_name: file path or file name string (e.g. './data/file.txt')
        d_type: data type for data or columns, e.g. {‘a’: np.float64, ‘b’: np.int32, ‘c’: ‘Int64’}
    '''
    if not isinstance(file_name,str):
        print(_ERROR_+' file_name '+f'{file_name}')
        sys.exit(_STR_ERROR_)
    if not isinstance(skip_rows,int):
        print(_ERROR_+' skip_rows '+f'{skip_rows}')
        sys.exit(_INT_ERROR_)
    if use_cols:
        if not isinstance(use_cols,list):
            print(_ERROR_+' use_cols '+f'{use_cols}')
            sys.exit(_LIST_ERROR_)
        if not all(isinstance(i,int) for i in use_cols):
            print(_ERROR_+' element of use_cols '+f'{use_cols}')
            sys.exit(_INT_ERROR_)
    if not isinstance(d_type,str):
        print(_ERROR_+' d_type '+f'{d_type}')
        sys.exit(_STR_ERROR_)
    if not isinstance(skip_blank,bool):
        print(_ERROR_+' skip_blank '+f'{skip_blank}')
        sys.exit(_BOOL_ERROR_)
    if not isinstance(clean_data,bool):
        print(_ERROR_+' clean_data '+f'{clean_data}')
        sys.exit(_BOOL_ERROR_)

    data_read = file_name+'.txt'
    if use_cols:
        data_temp = pd.read_csv(file_name,
                                usecols=use_cols,
                                dtype=d_type,
                                skiprows=skip_rows,
                                skip_blank_lines=skip_blank,
                                delim_whitespace=True)
    else:
        data_temp = pd.read_csv(file_name,
                                dtype=d_type,
                                skiprows=skip_rows,
                                skip_blank_lines=skip_blank,
                                delim_whitespace=True)
    data_temp.to_csv(data_read,index=False)
    # clean data before use
    if clean_data:
        clean_data_by_rules(data_read)
    data = np.loadtxt(data_read,delimiter=',')

    # delete temporary data read file
    if os.path.exists(data_read):
        try:
            os.remove(data_read)
        except PermissionError:
            print('No permission to delete the temporary file <'+data_read+'>')
        except:
            print('Undefined error occurred to delete the temporary file <'+data_read+'>')

    return data

def clean_data_by_rules(file_name):
    ''' replace unnecessary strings in data file with nan for matplotlib
        input file should contain text data
    '''
    with open(file_name, 'r') as file :
        file_data = file.read()

    # add more rules if needed
    file_data = file_data.replace('#,z','nan,nan') # replace '#,z'
    file_data = file_data.replace(',\n','nan,nan\n') # replace ',\n'

    with open(file_name, 'w') as file:
        file.write(file_data)

def find_file(dir_path,file_name,sort=True):
    """ find the file from provided directory
        (1) dir_path should be a string
        (2) file_name could be a string or a list
    """
    if not isinstance(dir_path,str):
        print(_ERROR_+' dir_path '+f'{dir_path}')
        sys.exit(_STR_ERROR_)

    # a directory can be a string or a list containing strings
    if not isinstance(file_name,(str,list)):
        print(_ERROR_+' file_name '+f'{dir_path}')
        sys.exit(_STR_LIST_ERROR_)

    # change a string to a list
    if not isinstance(file_name,list):
        file_name = [file_name]

    # avoid non-string element in a list
    if not all(isinstance(i,str) for i in file_name):
        print(_ERROR_+' file_name '+f'{dir_path}')
        sys.exit(_STR_LIST_ERROR_)

    if not isinstance(sort,bool):
        print(_ERROR_+' sort '+f'{sort}')
        sys.exit(_BOOL_ERROR_)

    path_name_list = []
    for i in range(len(file_name)):
        path_name_list.append(os.path.join(dir_path,file_name[i]))

    file_list = []
    for i in range(len(path_name_list)):
        file_list += [os.path.basename(x) for x in glob.glob(path_name_list[i])]

    if sort:
        file_list.sort()

    if len(file_list) != 0:
        return file_list
    else:
        sys.exit(_ERROR_+' no file found in <'+dir_path+'>')
##==========~~~~~~~~~~==========~~~~~~~~~~==========~~~~~~~~~~==========##


##==========~~~~~~~~~~==========~~~ERROR~~==========~~~~~~~~~~==========##
## DEFINE OUTPUT ERRORS
_ERROR_                 = 'Error:'

# ERROR SOLUTION
_INT_ERROR_             = 'Input Should be an Integer.'
_INT_SIGN_ERROR_        = 'Input Integer Should Non-Negative.'
_STR_ERROR_             = 'Input Should be a String.'
_INT_STR_ERROR_         = 'Input Should be an Integer or a String.'
_FLOAT_ERROR_           = 'Input Should be an Float Number.'
_INT_FLOAT_ERROR_       = 'Input Should be an Integer or a Float Number.'
_LIST_ERROR_            = 'Input Should be a List.'
_STR_LIST_ERROR_        = 'Input Should be a String or a List of Strings.'
_DICT_ERROR_            = 'Input Should be a Dictionary.'
_DICT_KEY_ERROR_        = 'Config Data Missing.'
_BOOL_ERROR_            = 'Input Should be a Boolean.'
##==========~~~~~~~~~~==========~~~~~~~~~~==========~~~~~~~~~~==========##


"""
Signal processing (:mod:`acous.signal`)
=======================================

Wavelets
--------


Spectral analysis
-----------------
   periodogram - Compute a (modified) periodogram.
   welch - Compute a periodogram using Welch's method.

"""
from . import windows
from .spectral import *


"""
Window functions (:mod:`scipy.signal.windows`)
==============================================

The suite of window functions for filtering and spectral estimation.

Window functions
----------------
   hanning - Hann window

"""
from .windows import *


from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='acous',
    version='0.0.1',
    description='Acoustics computing package for Python',
    author='Wende Li',
    author_email='wendeli@outlook.com',
    url='https://github.com/wendeli/acous',
    license='MIT',
    packages=find_packages(exclude=('tests', 'docs')),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
)


"""
Plot setup (:mod:`acous.plot`)
==============================

Plot setup functions
--------------------
   plot_setup - Define font size of generated pictures.

Helper routines
---------------
   read_data - Read data from a file

"""
from .plot_setup import *


import numpy as np
import scipy as sp

def fft(a, method='dft', positive_freq_only=False):
    """
    Compute the one-dimensional discrete Fourier Transform. The data is assumed to be
    uniformly/evenly sampled (so the sample interval is not used in the transform process),
    if not, please refer to the Nonequispaced Fast Fourier Transforms (NFFT) project (https://github.com/jakevdp/nfft).

    This function computes the one-dimensional n-point discrete Fourier
    Transform (DFT) with the efficient lower than the Fast Fourier Transform (FFT) algorithm.
    Use Numpy or Scipy if need true FFT with higher calculation efficiency.

    Parameters
    ----------
        a : array_like
            Input array, can be complex.
        method : {'dft', 'numpy', 'scipy'}, optional
        positive_freq_only : bool
            If only output the results corresponding to positive frequency values.

    Returns
    -------
        out : complex ndarray

    Notes
    -----
        FFT (Fast Fourier Transform) refers to a way the discrete Fourier
        Transform (DFT) can be calculated efficiently, by using symmetries in the
        calculated terms. The symmetry is highest when `n` is a power of 2, and
        the transform is therefore most efficient for these sizes.

        The values in the result follow so-called “standard” order: If A = fft(a, n),
        then A[0] contains the zero-frequency term (the sum of the signal), which is
        always purely real for real inputs. Then A[1:n/2] contains the positive-frequency
        terms, and A[n/2+1:] contains the negative-frequency terms, in order of
        decreasingly negative frequency. For an even number of input points, A[n/2]
        represents both positive and negative Nyquist frequency, and is also purely
        real for real input. For an odd number of input points, A[(n-1)/2] contains
        the largest positive frequency, while A[(n+1)/2] contains the largest negative
        frequency. The routine np.fft.fftfreq(n) returns an array giving the frequencies
        of corresponding elements in the output. The routine np.fft.fftshift(A) shifts
        transforms and their frequencies to put the zero-frequency components in the
        middle, and np.fft.ifftshift(A) undoes that shift. # ref [1].

        When the input a is a time-domain signal and A = fft(a), np.abs(A) is its
        amplitude spectrum and np.abs(A)**2 is its power spectrum.
        The phase spectrum is obtained by np.angle(A).

    References
    ----------
        [1] https://numpy.org/doc/stable/reference/routines.fft.html
        [2] https://numpy.org/doc/stable/reference/generated/numpy.fft.fft.html
        [3] https://jakevdp.github.io/blog/2013/08/28/understanding-the-fft/
        [4] https://rosettacode.org/wiki/Fast_Fourier_transform

    """
    if not isinstance(positive_freq_only, bool):
        raise ValueError("positive_freq_only should be an bool")

    a = np.asarray(a)
    N_pos = ((len(a)-1)//2)+1 # number of total sampling points corresponding to positive frequency values

    def is_power_of_two(x):
        """
        Function to check if x is power of 2
        If x & (x-1) == 0 return True else return False
        """
        if x < 1:
            return False
        elif (x & (x-1)):
            return False
        else:
            return True

    def dft(a):
        """
        My own DFT solver, intuitive but not efficient
        """
        N = len(a)
        A = np.empty(N, complex)
        for k in range(0,N):
            summ = complex(0.0,0.0)
            for n in range(0,N):
                summ += a[n]*(np.cos(2.0*np.pi*k*n/N)-1j*np.sin(2.0*np.pi*k*n/N))
            A[k] = round(summ.real,8)+round(summ.imag,8)*1j # set calculation precision

        return A
    
    def dft_faster(a):
        """
        Ref: https://rosettacode.org/wiki/Fast_Fourier_transform
        """
        N = len(a)
        A = np.empty(N, complex)
        n = np.arange(N)
        k = n.reshape(N,1)
        M = np.exp(-2j*np.pi*k*n/N)
        prod = np.dot(M, a)
        for k in range(N):
            A[k] = round(prod[k].real,8)+round(prod[k].imag,8)*1j # set calculation precision

        return A

    def naive_fourier_transform(a):
        """
        Naive implementation of the DFT equation.
        Ref: https://nbviewer.org/github/spatialaudio/communication-acoustics-exercises/blob/master/dft-solutions.ipynb
        The input must be a one-dimensional array.
        WARNING: this implementation is extremely inefficient, but it is still faster than my dft()
        """
        N = len(a)
        n = np.arange(N)

        return np.array([np.sum(a*np.exp(-1j*2*np.pi*k*n/N)) for k in range(N)])

    def fft_raw(a): # Cooley–Tukey FFT (https://rosettacode.org/wiki/Fast_Fourier_transform)
        N = len(a)
        if N <= 2:
            return dft(a)
        if not is_power_of_two(N): # N must be a even number, and FFT is fastest when N is the power of 2 (i.e. N = 2^n)
            return dft(a)
        A = np.empty(N, complex)
        A_even = fft_raw(a[0::2])
        A_odd = fft_raw(a[1::2])
        for k in range(N//2):
            expo = np.exp(-2j*np.pi*k/N)
            A[k] = A_even[k]+expo*A_odd[k]
            A[k+N//2] = A_even[k]-expo*A_odd[k]

        return A

    if method == 'dft':
        A = dft(a)
        #A = naive_fourier_transform(a)
        if positive_freq_only:
            return A[:N_pos]
        return A

    elif method == 'dft_faster':
        A = dft_faster(a)
        if positive_freq_only:
            return A[:N_pos]
        return A

    elif method == 'fft':
        A = fft_raw(a)
        if positive_freq_only:
            return A[:N_pos]
        return A

    elif method == 'numpy':
        if positive_freq_only:
            return np.fft.fft(a)[:N_pos]
        return np.fft.fft(a)

    elif method == 'scipy':
        if positive_freq_only:
            return sp.fft.fft(a)[:N_pos]
        return sp.fft.fft(a)

    raise ValueError(f"Invalid method value '{method}'; should be 'dft', 'numpy' or 'scipy'.")

def fftfreq(n, d=1.0, method='dft', positive_freq_only=False):
    """
    Return the Discrete Fourier Transform sample frequencies.

    The returned float array `f` contains the frequency bin centers in cycles
    per unit of the sample spacing (with zero at the start). For instance, if
    the sample spacing is in seconds, then the frequency unit is cycles/second.

    Given a window length `n` and a sample spacing `d`::
      f = [0, 1, ...,   n/2-1,     -n/2, ..., -1] / (d*n)  if n is even
      f = [0, 1, ..., (n-1)/2, -(n-1)/2, ..., -1] / (d*n)  if n is odd

    Parameters
    ----------
        n : int
            Window length.
        d : scalar, optional
            Sample interval (inverse of the sampling rate). Defaults to 1.0.
        method : {'dft', 'numpy', 'scipy'}, optional
        positive_freq_only : bool
            If only output the results corresponding to positive frequency values.

    Returns
    -------
        f : ndarray
            Array of length `n` containing the sample frequencies.

    References
    ----------
        [1] https://numpy.org/doc/stable/reference/generated/numpy.fft.fftfreq.html

    Examples
    --------
        >>> signal = np.array([-2, 8, 6, 4, 1, 0, 3, 5], dtype=float)
        >>> fourier = fft.fft(signal)
        >>> n = signal.size
        >>> timestep = 0.1
        >>> freq = fft.fftfreq(n, d=timestep)
        >>> freq
        array([ 0., 1.25, 2.5, 3.75, -5, -3.75, -2.5, -1.25])

    """
    if not isinstance(n, int):
	    raise ValueError("n should be an integer")

    if not isinstance(positive_freq_only, bool):
        raise ValueError("positive_freq_only should be an bool")

    N = n # number of total sampling points
    N_pos = ((N-1)//2)+1 # number of total sampling points corresponding to positive frequency values

    sampling_interval = d # sampling interval of the data points, if data is time series then unit is [sec]
    sampling_rate = 1.0/sampling_interval

    T = N*sampling_interval # physical duration of the data series, if data is time series then unit is [sec]
    freq_interval = 1.0/T # frequency resolution (frequency interval), = 1.0/(n*d) = sampling_rate/N

    if method == 'dft':
        if positive_freq_only:
            sampling_point_num = np.arange(0, N_pos, dtype=int)
        else:
            sampling_point_num = np.empty(N, int) # return a new array of given shape and type, without initializing entries.

            p1 = np.arange(0, N_pos, dtype=int)
            sampling_point_num[:N_pos] = p1

            p2 = np.arange(-(N//2), 0, dtype=int)
            sampling_point_num[N_pos:] = p2

        return sampling_point_num*freq_interval

    elif method == 'numpy':
        if positive_freq_only:
            return np.fft.fftfreq(n, d)[:N_pos]
        return np.fft.fftfreq(n, d)

    elif method == 'scipy':
        if positive_freq_only:
            return sp.fft.fftfreq(n, d)[:N_pos]
        return sp.fft.fftfreq(n, d)

    raise ValueError(f"Invalid method value '{method}; should be 'dft', 'numpy' or 'scipy'.")

def ifft(a, method='dft'):
    """
    Compute the one-dimensional inverse discrete Fourier Transform.

    This function computes the inverse of the one-dimensional *n*-point
    discrete Fourier transform computed by `fft`. In other words,
    ``ifft(fft(a)) == a`` to within numerical accuracy.
    For a general description of the algorithm and definitions,
    see `numpy.fft`.

    The input should be ordered in the same way as is returned by `fft`,
    i.e.,
    * ``a[0]`` should contain the zero frequency term,
    * ``a[1:n//2]`` should contain the positive-frequency terms,
    * ``a[n//2 + 1:]`` should contain the negative-frequency terms, in
      increasing order starting from the most negative frequency.

    For an even number of input points, ``A[n//2]`` represents the sum of
    the values at the positive and negative Nyquist frequencies, as the two
    are aliased together. See `numpy.fft` for details.

    Parameters
    ----------
        a : array_like
            Input array, can be complex.
        method : {'dft', 'numpy', 'scipy'}, optional

    Returns
    -------
        out : complex ndarray

    Notes
    -----
        FFT (Fast Fourier Transform) refers to a way the discrete Fourier
        Transform (DFT) can be calculated efficiently, by using symmetries in the
        calculated terms. The symmetry is highest when `n` is a power of 2, and
        the transform is therefore most efficient for these sizes.

    References
    ----------
        [1] https://numpy.org/doc/stable/reference/generated/numpy.fft.ifft.html

    """
    a = np.asarray(a)
    N = len(a)

    if method == 'dft':
        A = np.empty(N, complex)
        for k in range(0,N):
            summ = 0
            for n in range(0,N):
                summ += a[n]*(np.cos(2.0*np.pi*k*n/N)+1j*np.sin(2.0*np.pi*k*n/N))
            A[k] = ((round(summ.real, 8)+0.0) + (round(summ.imag, 8)+0.0)*1j)/N # set calculation precision

        return A

    elif method == 'numpy':
        return np.fft.ifft(a)

    elif method == 'scipy':
        return sp.fft.ifft(a)

    raise ValueError(f"Invalid method value '{method}'; should be 'dft', 'numpy' or 'scipy'.")


import numpy as np
from acous import fft

P_REF = 2.0e-5 # reference sound pressure based on the auditory threshold, [Pa]

def from_pressure_to_spl(p, pref=P_REF):
    """
    Calculate sound pressure level from sound pressure.

    Parameters
    ----------
        p : array_like
            Sound pressure or root mean square of sound pressure, [Pa]
        pref : scalar, optional
            Reference sound pressure, [Pa]

    Returns
    -------
        out : ndarray
            Sound pressure level, [dB]

    """
    p = np.asarray(p, dtype='float') # convert the input to an array

    return 20.0*np.log10(p/pref)

def from_spl_to_pressure(s, pref=P_REF):
    """
    Calculate sound pressure from sound pressure level.

    Parameters
    ----------
        s : array_like
            Sound pressure level, [dB]
        pref : scalar, optional
            Reference sound pressure, [dB]

    Returns
    -------
        out : ndarray
            Root mean square of sound pressure, [Pa]

    """
    s = np.asarray(s, dtype='float')

    return pref*np.power(10,s/20.0)

def cal_root_sum_square(p):
    """
    Calculate root sum square value from a sound pressure series

    Parameters
    ----------
        p : array_like
            Sound pressure or root mean square of sound pressure, [Pa]

    Returns
    -------
        out : scalar
            Root sum square of sound pressure, [Pa]

    """
    p = np.asarray(p, dtype='float')

    return np.sqrt(np.sum(np.square(p)))

def from_spl_to_oaspl(s, pref=P_REF):
    """
    Calculate overall sound pressure level from sound pressure level.

    Parameters
        s : array_like
            Sound pressure level, [dB]
        pref: scalar, optional
            Reference sound pressure, [dB]
    Returns
        out : scalar
            Overall sound pressure level, [Pa]

    """
    s = np.asarray(s, dtype='float')
    p = from_spl_to_pressure(s,pref)
    prss = cal_root_sum_square(p)

    return from_pressure_to_spl(prss,pref)

if __name__ == "__main__":
    pressure_rms = [20*1.123,200]
    spl = from_pressure_to_spl(pressure_rms)
    print(f'sound pressure level: {spl}')

    pressure = from_spl_to_pressure(spl)
    print(f'sound pressure: {pressure}')


"""
Discrete Fourier Transform (:mod:`acous.fft`)
=============================================

Standard FFTs
-------------
   fft - Discrete Fourier transform.
   ifft - Inverse discrete Fourier transform.

Helper routines
---------------
   fftfreq - Discrete Fourier Transform sample frequencies.

"""
from .basic import *